


#ifndef _FLASH_H_
#define _FLASH_H_

#include "parameters.h"
//#include "MPC5675K.h"
//#include "ssd_c90lc.h"


/************************************************************/
/* Flash defines 											*/
/************************************************************/
#define	INTERNAL_MEMORY_BASE	0
#define MAIN_ARRAY_BASE			INTERNAL_MEMORY_BASE
#define FLASH_CTL_REG_BASE		0xC3F88000				// Control Registers Base Address
#define FLASH_CTL_REG_BASE_1	0xC3FB0000				// Control Registers Base Address
#define FLASH_CTL_REG_BASE_2	0xC3F8C000				// Data Flash - Control Registers Base Address

#define MAIN_AR0_SIZE			ONE_MB					// Size is defined as 1056k, which is 1024k + 16k test + 16k shadow
#define MAIN_AR1_SIZE			ONE_MB
#define MAIN_AR2_SIZE			BLK_64KB				// Data flash is 72Kb including 8Kb Test Flash

#define MAIN_ARRAYSIZE    		0x200000	  			// main array size - 2.0 MB
#define NUM_LAS_BLKS      		16              		// Total number of LAS blks, non-interleaved block total
#define NUM_MAS_BLKS      		2              			// Total number of MAS blks, interleaved block total flash0 + flash1
#define NUM_HAS_BLKS      		4              			// Total number of HAS blks, interleaved block total flash0 + flash1
#define TOTAL_BLOCKS			NUM_LAS_BLKS+NUM_MAS_BLKS+NUM_HAS_BLKS

#define NUM_MODULES				2						// 2 code flash modules
#define	FLASH_ARRAY_BASE		0						// 1Mb Komodo Flash Array Base Address
#define	FLASH_ARRAY1_BASE		0x040000				// 1Mb Komodo Flash Array Base Address
#define OFFSET_MID_BLOCK0		0x080000				// Start of Mid Address Space
#define OFFSET_HIGH_BLOCK0		0x100000				// Start of High Address Space
#define	FLASH_ARRAY2_BASE		0x800000				// 72Kb Komodo Flash Array Base Address
#define DFLASH_ARRAY_BASE		0x800000				// Data Flash Base address

// Shadow rows for each 1Mb module 
#define SHADOW_BASE		  		0x200000				// first shadow base
#define SHADOW_SIZE       		BLK_16KB				// size of shadow row
#define SHADOW_END				(SHADOW_BASE + SHADOW_SIZE)
#define SHADOW_BK1_BASE			0x280000				// second shadow base
#define SHADOW_BK1_END			(SHADOW_BK1_BASE + SHADOW_SIZE)
#define PAGE_SIZE        		8             			//Flash page size in bytes 
#define BUFFER_SIZE_IN_BYTES	8						//Buffer size

//Test rows defined for each module - flash0, flash1, dataflash
#define TESTROW_BASE      		0x400000  				// flash0 Test Row base address
#define TESTROW_AR1_BASE		0x480000				// flash1 Test Row base address
#define TESTROW_AR2_BASE		0xC00000				// data flash Test Row base address
#define TESTROW_SIZE      		BLK_16KB        		// size of test row
#define TESTROW_SIZE_OTP		BLK_8KB					// size of usable OTP user area
#define TESTROW_END				TESTROW_BASE + TESTROW_SIZE	
#define TESTROW_END_OTP			TESTROW_BASE + TESTROW_SIZE_OTP	//user mode usable test space

#define HIGH_BLOCK0_SIZE		BLK_128KB
#define DFLASH_BLOCK0_SIZE		BLK_16KB

/* SOC shadow configuration */
// serial pass word and its page addr in shadow row 
#define PASS_ADDR				(SHADOW_BASE + 0x3DD8)
#define PASS_WORD0				0xFEEDFACE
#define PASS_WORD1				0xCAFEBEEF

//Flash censor word and its page addr - 64 bits - 55AA55AA_55AA55AA
#define CENSOR_ADDR				(SHADOW_BASE + 0x3DE0)
#define CENSOR_WORD				0x55AA55AA

//High Mid Block NVM data
#define HML_ADDR				(SHADOW_BASE + 0x3DF0)
#define PFBIU_ADDR				(SHADOW_BASE + 0x3E10)
#define USER_WORD_ADDR			(SHADOW_BASE + 0x3E18)
#define PFBIU1_ADDR				(SHADOW_BASE + 0x3E20)

//SOC configuration info stored in shadow
#define STCU_DATA_ADDR			(SHADOW_BASE + 0x10)
#define STCU_DATA_SIZE_BYTES	160	//0xA0
#define STCU_DATA_SIZE			(STCU_DATA_SIZE_BYTES / 4)	//size in words

/* Shadow Offset 0x3E18 - 64 bits */
/* 0xFFBFFFFF_0xFFFFFFFF --> DPM Mode, Watchdog Enabled */
/* 0x7FBFFFFF_0xFFFFFFFF --> DPM Mode, Watchdog Disabled */
#define DPM_LOCATION		(SHADOW_BASE + 0x3E18) 

/* As referenced from end of Shadow Base */
//#define PASSKEY_ADR			SHADOW_BASE + SHADOW_SIZE - 0x0228		// 0xxxxxxxDD8	Start address of page containing pass key
//#define PASSKEY_ADR1		SHADOW_BASE + SHADOW_SIZE - 0x0224		// 0xxxxxxxDDC	Start address of page containing pass key
//#define CENSOR_ADR			SHADOW_BASE + SHADOW_SIZE - 0x0220		// 0xxxxxxxDE0	Start address of page containing censorship words and LML
//#define HML_ADR				SHADOW_BASE + SHADOW_SIZE - 0x0210		// 0xxxxxxxDF0	Start address of page containing HML and SLL data
//#define PFBIU_ADR			SHADOW_BASE + SHADOW_SIZE - 0x01F0		// 0xxxxxxxE10	Start address of page containing BIU reset words
//#define PFBIU1_ADR			SHADOW_BASE + SHADOW_SIZE - 0x01E0		// 0xxxxxxxE20	Start address of page containing BIU1 reset words
//#define BOOT_DATA_ADR		SHADOW_BASE + 0x10						// 0xxxxxxxx10  Start address of page containing STCU boot data

#define STCU_OFFSET			0x10
#define PASSKEY_OFFSET		0x3DD8
#define CENSOR_OFFSET		0x3DE0
#define HML_OFFSET			0x3DF0		//(0x3DF0 & (~(BUFFER_SIZE_IN_BYTE - 1)))
#define PFBIU_OFFSET		0x3E10		//(0x3E10 & (~(BUFFER_SIZE_IN_BYTE - 1)))
#define USER_WORD_OFFSET	0x3E18
#define PFBIU1_OFFSET		0x3E20		//(0x3E20 & (~(BUFFER_SIZE_IN_BYTE - 1)))
#define PE_OFFSET			0x0068
/*****************************************/
/*****************************************/
/*****************************************/
/*****************************************/



/* Bit definitions and macros for FLASH_MCR */
#define MCR_EHV                   (0x00000001)   /*  Enable High Voltage */
#define MCR_ESUS                  (0x00000002)   /*  Erase Suspend */
#define MCR_ERS                   (0x00000004)   /*  Erase */
#define MCR_PSUS                  (0x00000008)   /*  Program Suspend */
#define MCR_PGM                   (0x00000010)   /*  Program */
#define MCR_PEG                   (0x00000200)   /*  Program/Erase Good */
#define MCR_DONE                  (0x00000400)   /*  State Machine Status */
#define MCR_PEAS                  (0x00000800)   /*  Program/Erase Access Space */
#define MCR_EDC				  	  (0x80000000)	 /*  Single-bit correction */
#define MCR_RWE                   (0x00004000)   /*  Read While Write Event Error */
#define MCR_EER                   (0x00008000)   /*  ECC Event Error */ 
#define MCR_MAS                   (0x00010000)   /*  Mid Address Space Size */
#define MCR_LAS(x)                (((x)&0x7)<<20)   /*  Low Address Space Size */
#define MCR_SIZE(x)               (((x)&0x7)<<24)   /*  Array Space Size */

/* Bit definitions and macros for FLASH_LML */
#define LML_LLOCK(x)              ((x)&0x3FF)         /*  Low Address Block Lock */
#define LML_MLOCK(x)              (((x)&0x3)<<16)   /*  Mid Address Block Lock */
#define LML_SLOCK                 (0x00100000)   /*  Shadow Lock */
#define LML_LME                   (0x80000000)   /*  Low and Mid Address Lock Enable */

/* Bit definitions and macros for FLASH_HBL */
#define HBL_HLOCK(x)              ((x)&0x3F)    /*  High Address Space Block Lock */
#define HBL_HBE                   (0x80000000)   /*  High Address Lock Enable */

/* Bit definitions and macros for FLASH_SLL */
#define SLL_SLLOCK(x)             ((x)&0x3FF)    /*  Secondary Low Address Block Lock */
#define SLL_SMLOCK(x)             (((x)&0x3)<<16)   /*  Secondary Mid Address Block Lock */
#define SLL_SSLOCK                (0x00100000)   /*  Secondary Shadow Lock */
#define SLL_SLE                   (0x80000000)   /*  Secondary Low and Mid Address Lock Enable */

/* Bit definitions and macros for FLASH_LMS */
#define LMS_LSEL(x)               ((x)&0x3FF)    /*  Low Address Space Block Select */
#define LMS_MSEL(x)               (((x)&0x3)<<16)   /*  Mid Address Space Block Select */

/* Bit definitions and macros for FLASH_HBS */
#define LMS_HSEL(x)               ((x)&0x3F)    /*  High Address Space Block Select */

#define ADR_SAD					(0x80000000)	/* shadow address */

/* Bit definitions and macros for FLASH_BIU0 */
#define BIU0_BFEN                  (0x00000001)   /*  Line Read Buffer Enable */
#define BIU0_PFLIM(x)              (((x)&0x00000003)<<1)    /*  Prefetch Limit */
#define BIU0_IPFEN                 (0x00000010)   /*  Instruction Prefetch Enable */
#define BIU0_DPFEN                 (0x00000040)   /*  Data Prefetch Enable */
#define BIU0_RWSC(x)               (((x)&0x00000007)<<8)    /*  Read Wait State Control */
#define BIU0_WWSC(x)               (((x)&0x00000003)<<11)   /*  Write Wait State Control */
#define BIU0_APC(x)                (((x)&0x00000007)<<13)   /*  Address Pipelining Control */
#define BIU0_M0PFE                 (0x00010000)   /*  Core Master Pre-fetch Enable */
#define BIU0_M1PFE                 (0x00020000)   /*  DMA Master Pre-fetch Enable */
#define BIU0_MxPFE(x)              (((x)&0x00003FFF)<<18)   /*  Master x Pre-fetch Enable (not used) */

/* Bit definitions and macros for FLASH_BIU1 */
#define BIU1_M0AP(x)               (((x)&0x00000003)<<0)    /*  Core Access protection */
#define BIU1_M1AP(x)               (((x)&0x00000003)<<2)    /*  DMA Access protection */
#define BIU1_MxAP(x)               (((x)&0x0FFFFFFF)<<4)    /*  Master x Access protection (not used) */

/* Bit definitions and macros for FLASH_BIU2 */
#define BIU2_SHDACC(x)             (((x)&0x000000FF)<<0)    /*  Shadow Data/Instruction Access */
#define BIU2_SHSACC(x)             (((x)&0x000000FF)<<8)    /*  Shadow Supervisor/User Access */
#define BIU2_MPAUNL(x)             (((x)&0x00000003)<<16)   /*  Enable Writes to the M0AP/M1AP bits in the PFAPR */
#define BIU2_DACCUNL               (0x00200000)   /*  Enable Writes to the PFDACC register */
#define BIU2_SACCUNL               (0x00400000)   /*  Enable Writes to the PFSACC register */
#define BIU2_CR1UNL                (0x00800000)   /*  Enable Writes to the PFCR1 register */
#define BIU2_LBCFG(x)              (((x)&0x00000003)<<30)   /*  Line Buffer Configuration */

/* Bit definitions and macros for FLASH_BIU3 */

/* Bit definitions and macros for FLASH_BIU4 */

/* Bit definitions and macros for FLASH_UT0 */
#define UT0_UTE                    (0x80000000)     /*  U-test enable */
#define UT0_SBCE			   	   (0x40000000)	    /*  Single-bit correction enable */
#define UT0_DSI(x)                 (((x)&0xFF)<<16) /*  Data syndrome input */
#define UT0_EIE                    (0x00000008)     /*  ECC data input enable */
#define UT0_AIS                    (0x00000004)     /*  Array integrity sequence */
#define UT0_AIE                    (0x00000002)     /*  Array integrity enable */
#define UT0_AID                    (0x00000001)     /*  Array integrity done */

#define FLASH_LMLR_PASSWORD 	0xA1A11111				/* password to unlock LML */
#define FLASH_HLR_PASSWORD 		0xB2B22222				/* password to unlock HBL */
#define FLASH_SLL_PASSWORD 		0xC3C33333				/* password to unlock SLL */
#define FLASH_UT_PASSWORD  		0xF9F99999				/* password to enable user testmode */

#define DEMO_PASS				0xA5A5A5A5
#define DEMO_FAIL				0xFAFAFAFA

#endif //_FLASH_H_