#include "MPC5643L.h"
#include "parameters.h"


#define FCCU_CFK_KEY 	0x618B7A50
#define FCCU_NCFK_KEY 	0xAB3498FE
extern void clear_CF(void);
extern void clear_NCF(void);

uint32_t SetSysClock(uint32_t ext_clk_mhz, uint32_t pll0_clk_mhz, uint32_t pll1_clk_mhz);
uint32_t result;

extern void DisableSoftwareWatchdog(void);

void InitializeModesAndClock(void) {
	if (ME.GS.B.S_CURRENT_MODE == 2) {
		clear_CF();
		clear_NCF();

		RGM.FES.R = 0xFFFF;     // Clear the FES
		RGM.FBRE.B.BE_SOFT_FUNC = 0; // RESET_B is asserted on a software 'functional' reset event.
		ME.MCTL.R = 0x00005AF0; // Change the mode to "reset" & Key
		ME.MCTL.R = 0x0000A50F; // Change the mode to "reset" & Key   	
  }
  
	ME.MER.R = 0x000025FF; // Enble all mode in mode entry MODE_ENABLE REGISTER

	ME.RUN[0].R = 0x001F0010;      //  Configure RUN 0 mode Optional
	ME.MCTL.R   = 0x40005AF0;      //  go to run 0 mode OPTIONAL
	ME.MCTL.R   = 0x4000A50F;      //  go to run 0 mode optional
	while(ME.GS.B.S_MTRANS==1){}      //  Wait for mode entry to complete
	while(ME.GS.B.S_CURRENT_MODE!=4){} //  Check RUN0 mode has been entered

	result=SetSysClock(EXT_CLK_MHZ, PLL0_TARGET_FREQ, PLL1_TARGET_FREQ);
	
	
	if(result)
		asm(nop);
	
}


void DisableSoftwareWatchdog (void) {
	// remove the SWT Soft lock
	SWT.SR.R = 0x0000C520;
	SWT.SR.R = 0x0000D928;
	// disable SWT
	SWT.CR.B.WEN = 0x0;
}

void clear_CF(void) {
	uint32_t i,a[4];
  
	for(i=0;i<4;i++) {
		FCCU.CFK.R = FCCU_CFK_KEY;
		FCCU.CF_S[i].R = 0xFFFFFFFF;

		while(FCCU.CTRL.B.OPS != 0x3){}           /* wait for the completion of the operation */
		a[i]=FCCU.CF_S[i].R;
	}
}

void clear_NCF(void) {
	uint32_t i,b[4];

	for(i=0;i<4;i++){
		FCCU.NCFK.R = FCCU_NCFK_KEY;
		FCCU.NCF_S[i].R = 0xFFFFFFFF;

		while(FCCU.CTRL.B.OPS != 0x3){}           /* wait for the completion of the operation */
		b[i]=FCCU.NCF_S[i].R;
	}
}
/*************************************************************************************************************************/                    	
int main(void) 
{
	unsigned long u32Counter;
	
	DisableSoftwareWatchdog();
	InitializeModesAndClock();


	


	/* Output pins */
	SIU.PCR[54].R = 0x0200;		// D4 green
	SIU.PCR[107].R = 0x0200; 	// D4 Red	
	SIU.PCR[106].R = 0x0200; 	// D1	
	SIU.PCR[83].R = 0x0200; 	// D2


	SIU.PCR[51].R = 0x0200; 	// GPIO51
	SIU.PCR[18].R = 0x0200; 	// LIN0_TXD
	SIU.PCR[16].R = 0x0200; 	// FLEXCAN0_TXD
	

	/* Input pins */
	SIU.PCR[12].R = 0x0100; 	// SW1 (DSPI2_SOUT)
	SIU.PCR[13].R = 0x0100; 	// SW2 (DSPI2_SIN)
	

	SIU.GPDO[107].R = SIU.GPDO[107].R ^ 0x01; 	// D4 red


  /* Loop forever */
  	for (;;) 
  	{
		u32Counter++;
		
		SIU.GPDO[106].R = SIU.GPDI[12].R; 	// turn on LED1 when SW1 is pressed
		SIU.GPDO[83].R = SIU.GPDI[13].R; 	// turn on LED2 when SW2 is pressed
		SIU.GPDO[51].R = 0x01; 				// GPIO51 high level

		SIU.GPDO[18].R = SIU.GPDO[18].R ^ 0x01; // LIN0_TXD toogle
		SIU.GPDO[16].R = SIU.GPDO[16].R ^ 0x01; // FLEXCAN0_TXD toogle

		if(u32Counter==0x7FFFF)
		{
			SIU.GPDO[54].R = SIU.GPDO[54].R ^ 0x01; 	// D4 green
			SIU.GPDO[107].R = SIU.GPDO[107].R ^ 0x01; 	// D4 red
			u32Counter=0;
		}
  }
}


/*************************************************************************************************************************/                    	
void SetMode(uint8_t mode)
{ 
	uint32_t retry = 0, timeout=0x2FF;
	
	do
	{
		ME.MCTL.R = (mode << 28 | 0x00005AF0u);        	/* Mode & Key */
		ME.MCTL.R = (mode << 28 | 0x0000A50Fu);        	/* Mode & Key */
		while((ME.GS.B.S_MTRANS == 1) && (timeout--)){} 	/* Wait for mode entry to complete */
		retry++;
	} while ((ME.GS.B.S_CURRENT_MODE != mode) && (retry < 1000)); /* Check DRUN mode has been entered */	
}


/*************************************************************************************************************************/                    	
uint32_t ComputePLLFactors(uint32_t *odf, uint32_t *ndiv, uint32_t *idf, uint32_t ext_clk_mhz, uint32_t pll0_clk_mhz)
{
//	uint32_t vco, clk, odf_n, idf_n, ndiv_n, found = FALSE;
	uint32_t vco, clk, odf_n, idf_n, ndiv_n, found = 0;

	idf_n = 0;
	while((idf_n < 15) && !found)
	{
		// 32 <= ndiv <= 96 are valid for legacy.
		ndiv_n = 32;
		while((ndiv_n < 97) && !found)
		{
			// PLL VCO can't operate in outside of this range:
			// 256 MHz <= VCO <= 512 MHz
			vco = (ext_clk_mhz * (ndiv_n)) / (idf_n + 1);
			if((vco >= FVCO_MIN) && (vco <= FVCO_MAX) && (vco >= pll0_clk_mhz))
			{
				// looking for divider if pll0_clk_mhz < Fvco
				// divide by 2 (odf = 0b00), 4 (odf = 0b01), 8 (odf = 0b10) or 16 (odf = 0b11)
				// 0 <= odf <= 3
				odf_n = 0;
				while((odf_n < 4) && (!found))
				{
					// looking for odf
					// 2^(odf +1) is equivalent to (1 << (odf + 1))
					clk = (vco / (1 << (odf_n + 1))); 
					if(clk == pll0_clk_mhz)
					{
						*idf = idf_n; 
						*odf = odf_n; 
						*ndiv = ndiv_n;
						return (PASS);
					}
					odf_n++;
				}
			}
			ndiv_n++;
		}
		idf_n++;
	}
	
	return (FAIL);
}


uint32_t CurrentSysClk;


/*************************************************************************************************************************/                    	
uint32_t SetSysClock(uint32_t ext_clk_mhz, uint32_t pll0_clk_mhz, uint32_t pll1_clk_mhz)
{
	// Normal PLL Mode Frequency Equation
	// fsys = (fref * ndiv) / ((idf + 1) * (2^(odf+1))
	// vco = (fref * ndiv) / (idf + 1)
	// 256 MHz <= vco <= 512 MHz.
	// 8 MHz <= Fextal <= 40 MHz
	// Valid IDF[0:14] - divide by 1 to 15
	// Valid ODF[0:3]  - divide by 2, 4, 8 or 16
	// Valid NDIV[32:96] - divide by 32 to 96.

	uint32_t	idf, odf, ndiv;
	uint32_t	modperiod, incstep;
	uint32_t	status = FAIL, found;

	/* Create pointer instances to see memory map in debugger */
	volatile struct CGM_tag *ptrCGM;
	volatile struct SIU_tag *ptrSIU;
	volatile struct ME_tag *ptrME;


  	ptrCGM = (struct CGM_tag*)&CGM;
  	ptrSIU = (struct SIU_tag*)&SIU;
 	ptrME = (struct ME_tag*)&ME;
	
	/*********************************************/
	/* setup clock source and divide options 	*/
	/*********************************************/
	
	/* Enable progressive clock switching to gradually increase current consumption		*/
	/* This feature steps up output clock - div by 8, div by 4, div by 2, then Target. 	*/
	CGM.FMPLL[0].CR.B.EN_PLL_SW = 1;	
	CGM.FMPLL[1].CR.B.EN_PLL_SW = 1;

	/* Setup Clocks */			
	#if (EXT_CLK_MHZ == 16)
	
		/* Use 16M IRC clock for PLL0/PLL1 source since it's stable and clean */
		CGM.OSC_CTL.B.OSCBYP = 1;				// 1=bypass oscillator.  Use 16Mhz Input to PLL0 and PLL1
		CGM.AC3_SC.B.SELCTL = 0;				// PLL0 Reference clock  0=16M IRC, 1=4-40M crystal osc
		CGM.AC4_SC.B.SELCTL = 0;				// PLL1 Reference clock  0=16M IRC, 1=4-40M crystal osc				
	
	#elif (EXT_CLK_MHZ == 40)
									
		/* crystal oscillator is bypassed so whatever is on xtalout (extal) is what gets fed to PLL0/1 */
		CGM.OSC_CTL.B.OSCBYP = 0;				// 0=oscillator for root clock - 40 Mhz
		CGM.AC3_SC.B.SELCTL = 1;				// PLL0 Reference clock  0=16M IRC, 1=4-40M crystal osc
		//CGM.AC4_SC.B.SELCTL = 1;				// don't use 40Mhz input to PLL1 due to VCO range limitation.  Use 16Mhz instead.				
		CGM.AC4_SC.B.SELCTL = 0;				// PLL1 Reference clock  0=16M IRC, 1=4-40M crystal osc				
	#endif

	/* Check if we are in safe mode first.  */

	if (ME.GS.B.S_CURRENT_MODE == SAFE_MODE)
	{
		RGM.FES.B.F_FCCU_SAFE = 0x1;  	// Clear Safe Request
		SetMode(DRUN_MODE);				// Attempt to enter DRUN mode
		if (ME.GS.B.S_CURRENT_MODE == SAFE_MODE)
		{
			// if we can't change it then device fails 
			//Err.operation = DEVICE_SAFE_MODE;
			return status;	//status = FAIL
		}
	}	


	/************************************/
	/* Setup divide options  			*/
	/************************************/
	//if commented leave divide on but default is divide by 1
	CGM.SC_DC0.R = 0;					// Turn OFF Divide system clock and periph clock (perips run 1/2 of system)
	CGM.AC0_DC0.R = 0;					// Turn OFF Divide options for Aux0 clock divider
	CGM.AC1_DC0.R = 0;					// FlexRay Clock divier disabled=0
	CGM.AC2_DC0.R = 0;					// FlexCAN Clock divier disabled=0
	
	
	/********************************************************************************/
	/* Turn on clock source for system in specific run modes						*/
	/* Clock source must be turned ON before enabling PLL0/PLL1 					*/
	/* PXS30 can use 16Mhz IRC since we have option to use this as input to PLLx	*/
	/* Notes:																		*/
	/*																				*/
	/*		1. 		Setup System Clock to use 16Mhz IRC, Enable clock source 		*/
	/*				(IRC or XOSC) and enter Mode									*/
	/*		2.		Setup PLL, turn on PLL for Mode, re-enter Mode 					*/
	/*		3.		Setup System Clock to Use PLL0									*/
	/*		4.		Setup PLL1														*/
	/********************************************************************************/	
	found = TIMEOUT;
	#if (EXT_CLK_MHZ == 16)	
		
		/**********************************/
		/* Use 16Mhz IRC as source to PLL */
		/**********************************/		
		do
		{
			ME.DRUN.B.IRCOSCON = 1;				// enable IRC
			ME.DRUN.B.XOSCON = 1;				// need this on
			#if (SET_MODE == DRUN_MODE)
				ME.DRUN.B.SYSCLK  = 0;			// Select internal IRC for now since PLL is not setup yet
			#elif (MODE == RUN0_MODE)
				ME.RUN[0].B.SYSCLK  = 0;		// Select internal IRC for now since PLL is not setup yet
			#endif
		  	SetMode (SET_MODE);	 					// RE enter the mode, to update the configuration
		  	
		} while((ME.GS.B.S_IRCOSC == 0) && (found--));   	// Wait for external osc to stabilize
	
	#elif (EXT_CLK_MHZ == 40)
	
		/**********************************/
		/* Use 40Mhz OSC as source to PLL0 */
		/**********************************/	
		do
		{
			ME.DRUN.B.XOSCON = 1;	 			// enable crystal OSC	
			#if (SET_MODE == DRUN_MODE)
				ME.DRUN.B.SYSCLK  = 0;			// Select internal IRC for now since PLL is not setup yet
			#elif (SET_MODE == RUN0_MODE)
				ME.RUN[0].B.SYSCLK  = 0;		// Select internal IRC for now since PLL is not setup yet
			#endif
		  	SetMode (SET_MODE);	 					// RE enter the mode, to update the configuration
		  	
		} while((ME.GS.B.S_XOSC == 0) && (found--));   	// Wait for external osc to stabilize
	
	#else
	
		return(0);
	
	#endif

	
	/**************************************************/
	/* Enable PLL after we setup mult/div factors */
	/**************************************************/
	// Make sure Fvco is within the min and max limit.
	// Valid IDF[0:14] which is divide by [1:15]
		
	if(found)
	{
		/* calculate idf, odf, and ndiv so we can setup PLL */
		found = ComputePLLFactors(&odf, &ndiv, &idf, ext_clk_mhz, pll0_clk_mhz);
		
		#if FM_ENABLED
		
			// Disable Frequency Modulation
			CGM.FMPLL[0].MR.B.FM_EN = 0;

			// Frequency Modulation Period
			// modperiod = round (fref / (4 x fmod))
			//	- fref represents the frequency of the feedback divider.
			//	- fmod represents the modulation frequency.
			// modperiod = EXT_CLK_MHZ / (4 x 24000)
			modperiod = (ext_clk_mhz * 1000) / (4 * MF_KHZ);
							
			// Increment Step
			// incstep = round ((( 2^15 - 1) x MD x EMFD) / (100 x 5 x modperiod))
			//	- md represents the peark modulation depth in percentage
			//	- EMFD represents the moninal value of the feeback loop divider
			// instep = (32767 * MD * odf)  / (500 * modperiod)
			incstep = (32767 * MOD * odf) / (500 * modperiod);
			
			CGM.FMPLL[0].MR.B.MOD_PERIOD = modperiod;
			CGM.FMPLL[0].MR.B.INC_STEP = incstep;
			
			// Enable Frequency Modulation
			CGM.FMPLL[0].MR.B.FM_EN = 1;	
			
		#endif			

		//	Write idf, odf, ndiv.  			
		CGM.FMPLL[0].CR.B.ODF = odf;
		CGM.FMPLL[0].CR.B.IDF = idf;
		CGM.FMPLL[0].CR.B.NDIV = ndiv;

		// Enable PLL now 
		found = TIMEOUT;
		do
		{		   	
	    	#if (SET_MODE == RUN0_MODE)
	    		ME.RUN[0].B.PLL0ON = 1;   		// Enable pll0
	    		ME.RUN[0].B.PLL1ON = 1;   		// Enable pll1
	 			SetMode (RUN0_MODE);	 		// RE enter the drun mode, to update the configuration
	 		#elif (SET_MODE == DRUN_MODE)
	 			ME.DRUN.B.PLL0ON = 1;   		// Enable pll0
	 			ME.DRUN.B.PLL1ON = 1;   		// Enable pll1
	 			SetMode (DRUN_MODE);	 		// RE enter the drun mode, to update the configuration
	 		#endif
	 		
		} while((ME.GS.B.S_PLL0 == 0) && (--found));   	// Wait for PLL0 stabilize	   

		if (found == 0)
		{
			return (PLL_ON_FAIL);
		}

		/* Wait for PLL to lock */
		found = TIMEOUT;						
		while((!CGM.FMPLL[0].CR.B.S_LOCK) && (--found)){}	
		if (found == 0)
		{
			return (PLL_LOCK_FAIL);
		}
		
		/* Only select PLL0 as primary system clock if is successfully changed and locked */
		#if SET_MODE == DRUN_MODE
	    	ME.DRUN.B.SYSCLK = SYS_PLL;				// Select PLL0 as primary system clock
			SetMode (DRUN_MODE);	 				// RE enter the drun mode, to update the configuration			
		#elif SET_MODE == RUN0_MODE
	    	ME.RUN[0].B.SYSCLK = SYS_PLL;			// Select PLL0 as primary system clock
			SetMode (RUN0_MODE);	 				// RE enter the drun mode, to update the configuration								
		#endif
		
		if(ME.GS.B.S_SYSCLK == SYS_PLL)
		{
			CurrentSysClk = pll0_clk_mhz;			// Assign global to carry current frequency for timer calculations
			status = PASS;
		}
		else
		{
			return (SET_SYS_CLK_FAIL); 
		}
	}	
	
	/********************************************************/
	/* Setup Second PLL 									*/
	/********************************************************/
	if (status == PASS)
	{
		#if (EXT_CLK_MHZ == 16)
		
			found = ComputePLLFactors(&odf, &ndiv, &idf, ext_clk_mhz, pll1_clk_mhz);
		
			// set 120 Mhz
			// idf, odf, ndiv = 1, 1, 60 gives pll2 @ 120 Mhz with 16Mhz input clock
			// sys = ext*ndiv / idf*odf --> 16M*60 / 2*4
			//idf = 1;		//div by 2
			//odf = 1;		//div by 4
			//ndiv = 60;		//pll loop division factor
			
			//set second pll to same settings for idf, odf, ndiv
			CGM.FMPLL[1].CR.B.ODF = odf;
			CGM.FMPLL[1].CR.B.IDF = idf;
			CGM.FMPLL[1].CR.B.NDIV = ndiv;
					
			/* Wait for PLL2 to lock */
			found = TIMEOUT;						// timeout waiting for FMPLL to lock
			while((!CGM.FMPLL[1].CR.B.S_LOCK) && (--found)){}	
			if (found == 0)
			{
				status = PLL2_LOCK_FAIL;
			}
			else
			{
				status = PASS;
			}		
					
		#elif (EXT_CLK_MHZ == 40)
			/* cannot use 40 Mhz source for PLL1 */
			/* ndiv minimum has to be 32 and that (32*40)=1280 violates VCO 256-512 Mhz range*/			
		#else			
			// Leave as default			
		#endif
	
	}
				
	return(status);	

}

