/*
 * parameters.h
 *
 *  Created on: May 10, 2011
 *      Author: r39626
 */

#ifndef PARAMETERS_H_
#define PARAMETERS_H_

//#include "typedefs.h"
//#include "MPC5675K.h"

/************************************************************/
/* PLL Defines												*/
/************************************************************/
#define	FVCO_MIN	256			// Min Fvco is 256 MHz
#define	FVCO_MAX	512			// Max Fvco is 512 MHz
#define MF_KHZ		24			// Modulation Frequency in 24 KHz
#define MOD			2			// Modulation dept +/-2%
//#define FM_ENABLED	1		// Enable Frequency Modulation
#ifndef	EXT_CLK_MHZ
	#define	EXT_CLK_MHZ		40			//	40 Mhz Oscillator for input clock
	//#define	EXT_CLK_MHZ			16			//	16Mhz IRC can be used as input to PLL0 and PLL1
	#define PLL0_TARGET_FREQ	180			//  180 Mhz for first PLL (PLL0)
	#define PLL1_TARGET_FREQ	120			//  120 Mhz for second PLL (PLL1)
#endif
#define TIMEOUT			0x200					// used for PLL lock timeout
#define SYS_PLL			4					// Value used in Clock X Select Control register within CGM
#define SEC_PLL_120		5	
#define SEC_PLL_90		8

/*******************/
/* General defines */
/*******************/
#define ONE_MB	     			0x00100000	            	//  1 MB
#define TWO_MB					0x00200000					//  2 MB
#define THREE_MB	            0x00300000	       	    	//  3 MB

#define BLK_8KB					0x00002000					// 8KB
#define BLK_16KB				0x00004000					// 16KB
#define BLK_32KB				0x00008000					// 32KB
#define BLK_48KB   				0x0000C000					// 48KB
#define BLK_64KB   				0x00010000					// 64KB
#define BLK_128KB 				0x00020000					// 128KB
#define BLK_256KB 				0x00040000					// 256KB
#define BLK_512KB				0x00080000					// 512KB

/* Define operating modes */
#define RESET_MODE	0x0
#define TEST_MODE	0x1
#define SAFE_MODE	0x2
#define DRUN_MODE	0x3
#define RUN0_MODE	0x4
#define RUN1_MODE	0x5
#define RUN2_MODE	0x6
#define RUN3_MODE	0x7
#define HALT_MODE	0x8
#define STOP_MODE	0xA
#define STBY_MODE	0xD	

/* FCCU Keys */
#define FCCU_CFK_KEY			0x618B7A50
#define FCCU_NCFK_KEY			0xAB3498FE
#define RGM_ENABLE				1				// Enable ME/RGM flag clearing
#define CRITICAL_FAULT			1				// For fault injection
#define NON_CRITICAL_FAULT		0				// For fault injection

/* Mode Entry Keys */
#define KEY1		0x5AF0
#define KEY2		0xA50F
#define SET_MODE	DRUN_MODE

/* Shadow Block DPM/LSM Modes */
#define LSM_MODE	1
#define DPM_MODE	0
#define LSM_ENABLE	0x00400000		//LSM bit in shadow flash

/* STCU Bypass or enabled Mode */
#define STCU_BYPASS			0xBB
#define STCU_ENABLED		0xEE

typedef struct
{
	uint32_t status;
	uint32_t data[2];
	uint32_t operation;
	uint32_t address;
	uint32_t bitcnt;
} ERROR;

#define	INIT_FLASH_FAIL				0x10000000
#define SET_SYS_CLK_FAIL			0x10000001	
#define PLL_LOCK_FAIL				0x10000002	
#define PLL2_LOCK_FAIL				0x10000022
#define PLL_ON_FAIL					0x10000003	
#define DEVICE_SAFE_MODE			0x10000004	
#define STCU_FAULT_FAIL				0x10000005	
#define INIT_SRAM_FAIL				0x10000006	

#define FAIL						0x01
#define	PASS						(!FAIL)
#define	TIMEOUT_ERROR				0x02
#define	INVALID_PARAMETER			0x04
#define	EER_ERROR_FLASH				0x18
#define	EDC_ERROR					0x10
#define	RWE_NOT_SET_ERROR			0x20
#define ERROR_INVALID_ADDRESS		0x40
#define	EER_ERROR_SRAM				0x80			// L2RAM double bit ECC Error
#define	SRAM_ECC_ERROR				0x80			// L2RAM double bit ECC Error
#define SRAM_EDC_ERROR				0x81			// SRAM Single bit ECC Error 
#define ERROR_WORD_ALIGNMENT		0x100
#define ERROR_DWORD_ALIGNMENT		0x200
#define RWE_NOT_CLEAR_ERROR			0x400			

#define	PAGE_SIZE_IN_BYTE			8				//	page size of flash in bytes.  
#define WORD_SIZE_IN_BYTE			4				//  word size in bytes

/* TWR-PXS30 LED Drivers */
#define LED00_ON		SIU.GPDO[48].R = 0;
#define LED00_OFF		SIU.GPDO[48].R = 1;
#define LED01_ON		SIU.GPDO[49].R = 0;
#define LED01_OFF		SIU.GPDO[49].R = 1;
#define LED2_ON			SIU.GPDO[60].R = 0;
#define LED2_OFF		SIU.GPDO[60].R = 1;
#define LED3_ON			SIU.GPDO[98].R = 0;
#define LED3_OFF		SIU.GPDO[98].R = 1;
#define LED00_ENABLE 	{ LED00_OFF; SIU.PCR[48].R = 0x200; }
#define LED01_ENABLE 	{ LED01_OFF; SIU.PCR[49].R = 0x200; }
#define LED2_ENABLE 	{ LED2_OFF; SIU.PCR[60].R = 0x200; }
#define LED3_ENABLE 	{ LED3_OFF; SIU.PCR[98].R = 0x200; }
#define BLINK_LED00		{ LED00_ON; wait_ms(200); LED00_OFF; }
#define BLINK_LED01		{ LED01_ON; wait_ms(200); LED01_OFF; }
#define BLINK_LED2		{ LED2_ON; wait_ms(200); LED2_OFF; }
#define BLINK_LED3		{ LED3_ON; wait_ms(200); LED3_OFF; }

/* TWR-PXS30 ERIQ push buttons */
#define EXT_INT0_ENABLE	{ SIU.PCR[78].R = 0x100; }	//EIRQ26 (IRQ_M) - PA=0b00, IBE=0b1
#define EXT_INT1_ENABLE	{ SIU.PCR[93].R = 0x100; }	//ERIQ31 (IRQ_C) - PA=0b00, IBE=0b1
	
//	macro defintions for accessing addresses/registers
#define WriteAddrLong(addr, value)  	(*((VUINT32 *) (addr)) = (value))
#define WriteAddrShort(addr, value)		(*((VUINT16 *) (addr)) = (value))
#define WriteAddrByte(addr, value)		(*((VUINT8 *)  (addr)) = (value))
#define ReadAddrLong(addr)  			(*((VUINT32 *) (addr)))
#define ReadAddrLongLong(addr)			(*((VUINT64 *) (addr)))
#define ReadAddrShort(addr)  			(*((VUINT16 *) (addr)))
#define ReadAddrByte(addr)  			(*((VUINT8 *)  (addr)))
#define OrAddrLong(addr, value)   		(*((VUINT32 *) (addr)) |= (value))
#define OrAddrShort(addr, value)   		(*((VUINT16 *) (addr)) |= (value))
#define AndAddrLong(addr, value)   		(*((VUINT32 *) (addr)) &= (value))
#define AndAddrShort(addr, value)   	(*((VUINT16 *) (addr)) &= (value))

#if __PPC_VLE__
	#if __option(vle)
		#define VLE_ON	1
		#pragma ppc_asm_to_vle on
	#else
		#define VLE_ON	0
	#endif
#else
	#define VLE_ON	0
#endif

#endif /* PARAMETERS_H_ */
